from moviepy.editor import *
from bgeditor.common.image_helper import create_virgin_layer
from bgeditor.common.utils import *
import os, json, textwrap, requests
os.environ["IMAGEMAGICK_BINARY"] = "magick"

class Lyric:

    def __init__(self, lyric_json, font_url, font_size, color, duration, stroke_color, stroke_width, bg_color,
                 lyric_moving, fade_in, fade_out,
                 wrap_width= 30, w=1920, h=1080):
        self.lyric_json = lyric_json
        self.lyric_data = json.loads(self.lyric_json)
        self.font_url = font_url
        self.font_path=None
        self.font_size = font_size
        self.duration = duration
        self.color = color
        self.w = w # width of group textbox
        self.h = h # high of group textbox
        self.stroke_color = stroke_color
        self.stroke_width = stroke_width
        self.bg_color=bg_color
        self.wrap_width= wrap_width
        self.lyric_moving = lyric_moving
        self.fade_in = round(float(fade_in),1)
        self.fade_out = round(float(fade_out),1)
        if self.bg_color is None or self.bg_color == "":
            self.bg_color="transparent"
    def init(self):
        self.font_path = cache_file(self.font_url)
        if self.font_path is None:
            return False
        self.normalnize()
        return True
    def optimize_font(self):
        wrapper = textwrap.TextWrapper(width=self.wrap_width)
        for lyric in self.lyric_data:
            lyric['line'] = wrapper.fill(text=lyric['line'])
        data_post = {}
        data_post['w'] = self.w
        data_post['font_url'] = self.font_url
        data_post['json_lyric'] = json.dumps(self.lyric_data)
        data_post['font_size_want'] = self.font_size
        data_post['wrap_width'] = self.wrap_width
        font_size_tmp = requests.post("http://db.automusic.win/music/lyric/font", json = data_post).text
        if font_size_tmp.isdigit():
            self.font_size = int(font_size_tmp)
            return True
        return False

    def normalnize(self):
        lyric_data_normal = []
        for lyric in self.lyric_data:
            try:
                if 'milliseconds' in lyric and 'line' in lyric and int(lyric['milliseconds']) >= 0 and lyric['line'] is not None and len(lyric['line'])>0:
                    lyric_data_normal.append(lyric)
            except:
                pass
        self.lyric_data=lyric_data_normal


    def make(self):
        arr_composite = []
        last_time = 0
        i = 0
        max_h = 0
        time_delay_fade=0
        for lyric in self.lyric_data:
            txt = TextClip(lyric['line'], font = self.font_path.replace("\\","/"), color=self.color, fontsize=self.font_size,
                           bg_color=self.bg_color, stroke_color=self.stroke_color, stroke_width=self.stroke_width, print_cmd= True)
            start_time = int(lyric['milliseconds']) / 1000
            if i < len(self.lyric_data) - 1:
                end_time = int(self.lyric_data[i + 1]['milliseconds']) / 1000
            else:
                end_time = start_time + 5
            w_txt, h_txt = txt.size

            if i % 2 == 0:
                last_h = h_txt
            else:
                if max_h < last_h + h_txt:
                    max_h = last_h + h_txt
            if self.lyric_moving:
                position = int((i % 2) * last_h)
            else:
                position = 0

            end_time += time_delay_fade # stay 1s
            comp = txt.set_position(("center", position)).set_duration(end_time - start_time).set_start(
                    start_time)
            if self.fade_in > 0 :
                comp= comp.crossfadein(self.fade_in)
            if self.fade_out > 0 :
                comp= comp.crossfadeout(self.fade_out)
            arr_composite.append(comp)
            last_time = end_time
            txt.close()
            i += 1
        if last_time < self.duration:
            txt = TextClip("...", font = self.font_path.replace("\\","/"), color=self.color, fontsize=self.font_size,
                           bg_color=self.bg_color, stroke_color=self.stroke_color, stroke_width=self.stroke_width)
            arr_composite.append(
                txt.set_position("center").set_duration(self.duration - last_time).set_start(
                    last_time).crossfadein(0.4).crossfadeout(0.4))
            txt.close()
        print('create vigrin with maxh : '+str(max_h))
        arr_composite.insert(0, ImageClip(create_virgin_layer(self.w, max_h))) #auto high of textlyric
        return CompositeVideoClip(arr_composite)






import youtube_dl, requests
from bgeditor.common.utils import get_dir, download_file,normal_audio
from bgeditor.dao.FFmpeg import create_loop_audio_times
import uuid,json, shutil,os
from moviepy.editor import *
import subprocess
import urllib
def download_audio(url,ext='mp3'):
    file_name = str(uuid.uuid4()) + "." + ext
    rs = os.path.join(get_dir('download'), file_name)
    ydl_opts = {
        'outtmpl': rs,
        'format': 'bestaudio/m4a',
    }
    with youtube_dl.YoutubeDL(ydl_opts) as ydl:
        ydl.download([url])
    return rs
def create_lyric_outro_songs(deezer_artist_id, lyric_outro_repeat, retries=3):
    if deezer_artist_id == 977996688:
        return None
    res = requests.get(f"http://source.automusic.win/deezer/artist/preview/{deezer_artist_id}/{lyric_outro_repeat}").json()
    arr_song=[]
    file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
    file_merg = open(file_merg_path, "a")
    final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp3')
    for item in res:
        tmp = download_file(item, ext='mp3')
        try:
            audio_test = AudioFileClip(tmp)
            audioduration = audio_test.duration
            audio_test.close()
            if audioduration < 1:
                continue
        except:
            continue
            pass
        tmp = normal_audio(tmp)
        tmp = fade_audio(tmp)
        arr_song.append(tmp)
        file_merg.write("file '%s'\n" % tmp)
    file_merg.close()
    if retries>0 and len(arr_song) != lyric_outro_repeat:
        retries-=1
        os.remove(file_merg_path)
        return create_lyric_outro_songs(deezer_artist_id, lyric_outro_repeat, retries=retries)

    if len(arr_song) > 1:
        cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
        os.system(cmd)
        os.remove(file_merg_path)
        for song in arr_song:
            try:
                os.remove(song)
            except:
                pass
    else:
        if len(arr_song) == 1:
            return arr_song[0]
    if len(arr_song) == 0:
        return None
    try:
        audio_moviepy = AudioFileClip(final_clip_path)
        if audio_moviepy.duration < 1:
            final_clip_path = None
        audio_moviepy.close()
    except:
        pass
    return final_clip_path
def fade_audio(audio_path, fade_time=3, dur=30):
    tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'),str(uuid.uuid4()) + '-' + os.path.basename(audio_path))
    duration = dur-fade_time
    cmd = f'ffmpeg -i "{audio_path}" -af "afade=t=in:st=0:d={fade_time}, afade=t=out:st={duration}:d={fade_time}" "{tmp_clip_path}"'
    process = subprocess.Popen(cmd, shell=True, stdout=subprocess.PIPE)
    process.wait()
    os.remove(audio_path)
    return tmp_clip_path
def create_compilation_songs(data, job_id, mf_server):
    #[{"type":3,"url":"","repeat":1}]
    #3: youtube_video
    #7: deezer
    #8: link direct
    arr_songs=data
    file_merg_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()))
    file_merg = open(file_merg_path, "a")
    final_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-final.mp3')
    cnt_song=0
    one_song_path=""
    try:
        for song in arr_songs:
            try:
                arr_tmp=song['uri'].split(":")
                song['local']=''
                if arr_tmp[0] == "youtube":#youtube
                    song['local']=download_audio(arr_tmp[2])
                if arr_tmp[0] == "direct":#direct
                    tmp=song['uri'].replace("direct:track:","")
                    ext = None
                    if "gdrive" in tmp:
                        ext="mp3"
                    song['local'] = download_file(tmp,ext=ext)
                if arr_tmp[0] == "deezer":#deezer
                    song_info=requests.get("http://source.automusic.win/deezer/track/get/"+arr_tmp[2], timeout=180).json()
                    song['local']=download_file(song_info['url_128'])
                if arr_tmp[0] == "spotify":  # spotify
                    arr_song_info = requests.get("http://source.automusic.win/spotify/track/get/" + arr_tmp[2], timeout= 180).json()
                    if len(arr_song_info)>0:
                        song_info = arr_song_info[0]
                        song['local'] = download_file(song_info['url_128'])
                if arr_tmp[0] == "soundcloud":
                    tmp = song['uri'].replace("soundcloud:track:", "")
                    song['local'] = download_file(tmp)
                if arr_tmp[0] == "automusic" and arr_tmp[1] == "source":
                    arr_song_info = requests.get("http://source.automusic.win/config/f-retrieve/"+arr_tmp[2] +
                                                 f"?jobid={job_id}&mfs={urllib.parse.quote_plus(mf_server)}", timeout= 600).json()
                    for song_info in arr_song_info:
                        try:
                            arr_songs.append({"uri":"direct:track:"+song_info['url_128'], "repeat":1})
                        except:
                            pass

                #after download song, re-check song
                try:
                    audio_test = AudioFileClip(song['local'])
                    audioduration = audio_test.duration
                    audio_test.close()
                    if audioduration < 1:
                        continue
                except:
                    continue
                    pass

                song['local']=normal_audio(song['local'])
                if song['repeat'] > 1:
                    song['local']= create_loop_audio_times(song['local'], song['repeat'])

                if not "coolbg_ffmpeg" in song['local']:
                    tmp_clip_path = os.path.join(get_dir('coolbg_ffmpeg'), str(uuid.uuid4()) + '-' + os.path.basename(song['local']))
                    shutil.copyfile(song['local'], tmp_clip_path)
                    os.remove(song['local'])
                    song['local']=tmp_clip_path
                one_song_path=song['local']
                file_merg.write("file '%s'\n" % song['local'])
                cnt_song+=1
            except:
                import traceback
                traceback.print_exc()
                pass
        file_merg.close()
        if cnt_song>1:
            cmd = "ffmpeg -y -f concat -safe 0 -i \"%s\" -codec copy \"%s\"" % (file_merg_path, final_clip_path)
            os.system(cmd)
            os.remove(file_merg_path)
            for song in arr_songs:
                try:
                    os.remove(song['local'])
                except:
                    pass
        else:
            if cnt_song ==1:
                return one_song_path
    except:
        pass
    if cnt_song == 0:
        return None
    try:
        audio_moviepy = AudioFileClip(final_clip_path)
        if audio_moviepy.duration < 1:
            final_clip_path=None
        audio_moviepy.close()
    except:
        pass
    return final_clip_path




